#!@PERL@

# --8<--8<--8<--8<--
#
# Copyright (C) 2010 Smithsonian Astrophysical Observatory
#
# This file is part of make-world
#
# make-world is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# make-world is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

package MakeWorld::Package::Cmake;

use strict;
use warnings;

use Carp;
use File::Spec::Functions qw/ catdir /;
use File::pushd qw[ pushd ];
use Cwd qw[ getcwd ];

use base 'MakeWorld::Package';

use MakeWorld::Global;
use MakeWorld::Run;
use MakeWorld::Utils qw[ find_exe ];
use File::Temp qw( tempdir );

sub matches_contents {  -f 'CMakeLists.txt' }

sub configure
{
    my $self = shift;

    $self->SUPER::configure && return;

    my $config_args = $self->{option}{configure}{arg};

    # just in case
    local %ENV = %ENV;

    # check for configure file. if its a dryrun, it's probable that the
    # source hasn't been unarchived and we won't see the configure file.
    # just assume its there. can't hurt!
    if ( -f 'CMakeLists.txt' || $Opt->dryrun )
    {
	my $source_dir = getcwd();
	my $build_dir = '.';

	if ( !$Opt->dryrun ) {

	    unlink( 'CMakeCache.txt' );
	    $build_dir = tempdir( DIR => $source_dir )
	      or die( "error creating temporary build directory\n" );
	    $self->set_option( {build_dir => $build_dir} );

	}

	my $cwd = pushd( $build_dir );

	run( find_exe( 'cmake' ),
	     '-DCMAKE_INSTALL_PREFIX=' . $Opt->prefix,
	     $config_args,
	     $source_dir,
	   )
	  or die( "error configuring package\n" );
    }
}

sub build
{
    my $self = shift;

    $self->SUPER::build && return;

    my $cwd = pushd( $self->option('build_dir') );

    run( $Opt->make_cmd  )
	or die( "error building package\n" );
}

sub test
{
    my $self = shift;

    my $cwd = pushd( $self->option('build_dir') );

    $self->SUPER::test ||
      die( "no test exe specified\n" );
}


sub install
{
    my $self = shift;

    $self->SUPER::install && return;

    my $cwd = pushd( $self->option('build_dir') );

    run( $Opt->make_cmd, 'install' )
      or die( "error installing package\n" );
}


1;
